#ifndef __JOBWIDGET_H
#define __JOBWIDGET_H

#include <QTreeWidget>
#include <QPointer>
#include "fly-admin-printer_export.h"

class QAction;
class QEvent;
class QTimer;
class SecurityJobAction;

namespace Cups
{
class Connection;
class Job;
}

class FLY_ADMIN_PRINTER_EXPORT JobWidget: public QTreeWidget
{
    Q_OBJECT
public:
    JobWidget(QWidget *parent = nullptr);
    ~JobWidget();
    void load(Cups::Connection *connection, const QString &printer = QString());

    QAction *m_aPlay;
    QAction *m_aAbort;
    QAction *m_aRestart;
    QAction *m_aMove;
    QAction *m_aAuthenticate;
    QAction *m_aPurge;
    QAction *m_aShowCompleted;
    QAction *m_aShowCanceled;
    QAction *m_aShowAborted;

public slots:
    virtual void currentItemChanged(QTreeWidgetItem*it, QTreeWidgetItem*);
    virtual void reload();
    virtual void cancelJob();
    virtual void purgeJob();
    virtual void playJob();
    virtual void restartJob();
    virtual void authenticateJob();

protected:
    virtual bool eventFilter(QObject *o, QEvent *e) override;
    void connectToDBus();
    void disconnectFromDBus();

protected slots:
    void onJobState(const QString &text, const QString &uri, const QString &name);

signals:
    void jobsStateChanged();

private:
    void configureItem(QTreeWidgetItem *item, const Cups::Job &job, const QList <QVariantMap> &dests, bool update);
    QTimer *m_refreshTimer;
    QList<SecurityJobAction*> m_extraActions;
    Cups::Connection *m_connection = nullptr;
    QString m_printer;
};

#endif
